@@ -38,32 +38,27 @@ choice
 	  that uClibc will work at all....
 
 	  Here are the available choices:
-	  - "SH2" Hitachi SH2
-	  - "SH3" Hitachi SH3
-	  - "SH4" Hitachi SH4
-	  - "SH5" SH-5 101, 103
+	  - "SH2" SuperH SH-2
+	  - "SH3" SuperH SH-3
+	  - "SH4" SuperH SH-4
+	  - "SH5" SuperH SH-5 101, 103
 
 config CONFIG_SH2
-	select ARCH_HAS_NO_FPU
 	select ARCH_HAS_NO_MMU
 	select ARCH_HAS_NO_LDSO
 	bool "SH2"
 
 config CONFIG_SH3
-	select HAS_FPU
 	select UCLIBC_HAS_MMU
-	select ARCH_HAS_NO_LDSO
 	bool "SH3"
 
 config CONFIG_SH4
-	select HAS_FPU
 	select UCLIBC_HAS_MMU
+	select FORCE_SHAREABLE_TEXT_SEGMENTS
 	bool "SH4"
 
 config CONFIG_SH5
-	select HAS_FPU
 	select UCLIBC_HAS_MMU
-	select ARCH_HAS_NO_LDSO
 	bool "SH5"
 
 endchoice
