@@ -88,14 +88,6 @@ _syscall3(int, __syscall_open, const char *, file, int, flags, __kernel_mode_t,
 int __libc_open (const char * file, int flags, ...)
 {
 	mode_t mode;
-	if (unlikely(file==NULL || *file=='\0')) {
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(file)+1) > NAME_MAX))) {
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
 	if (flags & O_CREAT) {
 		va_list ap;
 		va_start(ap, flags);
@@ -147,18 +139,7 @@ _syscall1(int, unlink, const char *, pathname);
 static inline
 _syscall3(int, __syscall_execve, const char *, filename, 
 		char *const *, argv, char *const *, envp);
-int execve(const char *filename, char *const *argv, char *const *envp)
-{ 
-	if (unlikely(filename==NULL || *filename=='\0')) {
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(filename)+1) > NAME_MAX))) {
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
-	return __syscall_execve(filename, argv, envp);
-}
+weak_alias(__syscall_execve, execve)
 #endif
 
 //#define __NR_chdir            12
@@ -168,18 +149,7 @@ int execve(const char *filename, char *const *argv, char *const *envp)
 #include <sys/param.h>
 static inline
 _syscall1(int, __syscall_chdir, const char *, path);
-int chdir(const char *path)
-{ 
-	if (unlikely(path==NULL || *path=='\0')) {
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
-	return __syscall_chdir(path);
-}
+weak_alias(__syscall_chdir, chdir)
 #endif
 
 //#define __NR_time             13
@@ -509,22 +479,7 @@ int kill(pid_t pid, int sig)
 #include <stdio.h>
 static inline
 _syscall2(int, __syscall_rename, const char *, oldpath, const char *, newpath);
-int rename(const char *oldpath, const char *newpath)
-{ 
-	if (unlikely((oldpath==NULL || newpath==NULL ||
-					*oldpath=='\0' || *newpath=='\0'))) 
-	{
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(oldpath)+1) >= NAME_MAX) || 
-				((strlen(newpath)+1) >= NAME_MAX))) 
-	{
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
-	return __syscall_rename(oldpath, newpath);
-}
+weak_alias(__syscall_rename, rename)
 #endif
 
 //#define __NR_mkdir            39
@@ -699,6 +654,9 @@ int __libc_fcntl(int fd, int cmd, ...)
 	return(__syscall_fcntl(fd, cmd, arg));
 }
 weak_alias(__libc_fcntl, fcntl)
+#if ! defined __NR_fcntl64 && defined __UCLIBC_HAS_LFS__
+weak_alias(__libc_fcntl, fcntl64);
+#endif
 #endif
 
 //#define __NR_mpx              56
@@ -740,18 +698,7 @@ mode_t umask(mode_t mode)
 #include <sys/param.h>
 static inline
 _syscall1(int, __syscall_chroot, const char *, path);
-int chroot(const char *path)
-{ 
-	if (unlikely(path==NULL || *path=='\0')) {
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
-	return __syscall_chroot(path);
-}
+weak_alias(__syscall_chroot, chroot)
 #endif
 
 //#define __NR_ustat            62
@@ -1173,18 +1120,7 @@ _syscall3(int, setpriority, __priority_which_t, which, id_t, who, int, prio);
 #include <sys/vfs.h>
 static inline
 _syscall2(int, __syscall_statfs, const char *, path, struct statfs *, buf);
-int statfs(const char *path, struct statfs *buf)
-{ 
-	if (unlikely(path==NULL || *path=='\0')) {
-		__set_errno(EINVAL);
-		return -1;
-	}
-	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
-		__set_errno(ENAMETOOLONG);
-		return -1;
-	}
-	return __syscall_statfs(path, buf);
-}
+weak_alias(__syscall_statfs, statfs)
 #endif
 
 //#define __NR_fstatfs          100
@@ -1413,6 +1349,7 @@ weak_alias(modify_ldt, __modify_ldt);
 #include <sys/timex.h>
 _syscall1(int, adjtimex, struct timex *, buf);
 weak_alias(adjtimex, __adjtimex);
+weak_alias(adjtimex, ntp_adjtime);
 #endif
 
 //#define __NR_mprotect         125
@@ -1423,11 +1360,29 @@ _syscall3(int, mprotect, void *, addr, size_t, len, int, prot);
 
 //#define __NR_sigprocmask      126
 #ifndef __NR_rt_sigprocmask
-#ifdef L_sigprocmask
+#ifdef L___syscall_sigprocmask
 #include <signal.h>
-#undef sigprocmask
-_syscall3(int, sigprocmask, int, how, const sigset_t *, set, 
+#define __NR___syscall_sigprocmask __NR_sigprocmask
+static inline
+_syscall3(int, __syscall_sigprocmask, int, how, const sigset_t *, set,
 		sigset_t *, oldset);
+#undef sigprocmask
+int sigprocmask(int how, const sigset_t *set, sigset_t *oldset)
+{
+	if (set &&
+#if (SIG_BLOCK == 0) && (SIG_UNBLOCK == 1) && (SIG_SETMASK == 2)
+	(((unsigned int) how) > 2)
+#else
+#warning "compile time assumption violated.. slow path..."
+	((how != SIG_BLOCK) && (how != SIG_UNBLOCK) && (how != SIG_SETMASK))
+#endif
+	   )
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return(__syscall_sigprocmask(how, set, oldset));
+}
 #endif
 #endif
 
@@ -1473,12 +1428,14 @@ _syscall4(int, quotactl, int, cmd, const char *, special , int, id, caddr_t, add
 //#define __NR_getpgid          132
 #ifdef L___syscall_getpgid
 #define __NR___syscall_getpgid __NR_getpgid
+#define __FAVOR_BSD
 static inline
 _syscall1(__kernel_pid_t, __syscall_getpgid, __kernel_pid_t, pid);
-pid_t getpgid(pid_t pid)
+pid_t __getpgid(pid_t pid)
 {
 	return(__syscall_getpgid(pid));
 }
+weak_alias(__getpgid, getpgid);
 #endif
 
 //#define __NR_fchdir           133
@@ -1936,7 +1893,19 @@ _syscall4(int, __rt_sigprocmask, int, how, const sigset_t *, set,
 
 int sigprocmask(int how, const sigset_t *set, sigset_t *oldset) 
 {
-	  return __rt_sigprocmask(how, set, oldset, _NSIG/8);
+	if (set &&
+#if (SIG_BLOCK == 0) && (SIG_UNBLOCK == 1) && (SIG_SETMASK == 2)
+			(((unsigned int) how) > 2)
+#else
+#warning "compile time assumption violated.. slow path..."
+			((how != SIG_BLOCK) && (how != SIG_UNBLOCK) && (how != SIG_SETMASK))
+#endif
+	   )
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
+	return __rt_sigprocmask(how, set, oldset, _NSIG/8);
 }
 #endif
 #endif
@@ -2229,10 +2198,6 @@ _syscall3(int, madvise, void*, __addr, size_t, __len, int, __advice);
 #define __NR___syscall_fcntl64 __NR_fcntl64
 static inline
 _syscall3(int, __syscall_fcntl64, int, fd, int, cmd, long, arg);
-#else
-#define __syscall_fcntl64  __syscall_fcntl
-extern int __syscall_fcntl(int fd, int cmd, ...);
-#endif
 int __libc_fcntl64(int fd, int cmd, ...)
 {
 	long arg;
@@ -2244,6 +2209,7 @@ int __libc_fcntl64(int fd, int cmd, ...)
 }
 weak_alias(__libc_fcntl64, fcntl64)
 #endif
+#endif
 
 //#define __NR_security		223	/* syscall for security modules */
 //#define __NR_gettid		224
